
from modules.basemodule import BaseModule
from modules.devops.docker import Docker
import os
class MongoDb(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def HandleMongoDb(self, db_options):
        mongo_docker_options = self.GetDockerOptions(db_options)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(db_options['name'], mongo_docker_options)
        docker_instance.AddVolume('mongodb-volume')
    
    def GetDockerOptions(self, db_options):
        mongo_docker_volume_dir = os.path.normpath(os.path.join(self.outputPath,'docker_volumes','mongodb'))
        if not os.path.isdir(mongo_docker_volume_dir):
            os.makedirs(mongo_docker_volume_dir)
        
        mongo_docker_options = {
            'image': db_options['name'].lower(),
            'container_name': db_options['name'],
            'volumes': ['mongodb-volume:'+mongo_docker_volume_dir],
            'ports':[],
            'restart': 'on-failure',
            'links':[],
            'depends_on':[],
            'networks': ['localnet']
        }
        # Add Ports
        if 'ports' in db_options:
            for port in db_options['ports']:
                mongo_docker_options['ports'].append(str(port)+':'+str(port))
        # Default Port if Not provided
        else:
            mongo_docker_options['ports'].append('"27017:27017"')
        mongo_using_services = self.FindMongoUsingServiceNames(db_options['name'])
        # Add Links So We can use mongo instance name to connect it in services
        for service_name in mongo_using_services:
            mongo_docker_options['links'].append(service_name)

        if 'docker_compose_override' in db_options:
            mongo_docker_options.update(db_options['docker_compose_override'])
        return mongo_docker_options

    def FindMongoUsingServiceNames(self,mongo_name):
        api_services = []
        if 'api_services' not in self.projectOptions:
            return api_services
        if len(self.projectOptions['api_services'])== 0:
            return api_services
        for service in self.projectOptions['api_services']:
            for key, value in service.items():            
                if 'database' in value:
                    if value['database']['provider'] == mongo_name:
                        api_services.append(value['name'])
        return api_services

def GetConnectionString(api_options,mongodb_options):
    """
    Returns ConnectionString for Mongodb
    """
    db_name = api_options['database']['database_name']
    db_host_dev = 'localhost'
    db_host = mongodb_options['name']
    db_username = mongodb_options['username']
    db_password = mongodb_options['password']
    conn_string = 'mongodb://{0}:{1}@{2}/{3}'.format(db_username,db_password,db_host,db_name)
    conn_string_dev=  'mongodb://{0}:{1}@{2}/{3}'.format(db_username,db_password,db_host_dev,db_name)
    return conn_string, conn_string_dev


from modules.basemodule import BaseModule
from modules.devops.docker import Docker
import os
import shutil
class Redis(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def HandleRedisDatabase(self, db_options):
        print (self.project_templates_paths)
        databasesPath = os.path.join(self.project_templates_paths,'databases')
        redis_template_folder = os.path.join(databasesPath,'redis')
        redis_project_folder = os.path.join(self.outputPath, db_options['name'])
        if os.path.isdir(redis_project_folder):
            shutil.rmtree(redis_project_folder,ignore_errors=True)
        #Copy template (config and Dockerfile)
        shutil.copytree(redis_template_folder,redis_project_folder)

        docker_options = self.GetRedisDockerOptions(db_options)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(db_options['name'],docker_options)
        docker_instance.AddVolume('redis-volume')

    def GetRedisDockerOptions(self, db_options):

        docker_volume_dir = os.path.normpath(os.path.join(self.outputPath,'docker_volumes','redis'))
        if not os.path.isdir(docker_volume_dir):
            os.makedirs(docker_volume_dir)

        redis_docker_options = {
            'image': db_options['name'].lower(),
            'build': {
                'context': db_options['name']+'/',
                'dockerfile': 'Dockerfile',
            },
            'container_name': db_options['name'],
            'ports':[],
            'restart': 'on-failure',
            'links':[],
            'depends_on':[],
            'networks': ['localnet'],
            'volumes': ['redis-volume:'+docker_volume_dir],
        }        
        # Add Ports
        if 'ports' in db_options:
            for port in db_options['ports']:
                redis_docker_options['ports'].append(str(port)+':'+str(port))
        # Default Port if Not provided
        else:
            redis_docker_options['ports'].append('"6379:6379"')
        redis_using_services = self.FindRedisUsingServiceNames(db_options['name'])
        # Add Links So We can use redis instance name to connect it in services
        for service_name in redis_using_services:
            redis_docker_options['links'].append(service_name)
        return redis_docker_options
    
    def FindRedisUsingServiceNames(self, redis_name):
        services = []
        for service in self.projectOptions['api_services']:
            for _, value in service.items():
                if 'cache' in value:
                    if value['cache']['type'] == 'redis':
                        services.append(value['name'])
        for identity_service in self.projectOptions['identity_services']:
            for _, value in identity_service.items():
                if  'cache' in value:
                    if value['cache']['type'] == 'redis':
                        services.append(value['name'])
        
        return services

def BuildRedisConnectionString(redis_options):
    return redis_options['name'], 'localhost:6379'



from modules.basemodule import BaseModule
from modules.devops.docker import Docker

from modules.utils.utils import (InDbQ,
    FindClientWithName,
    FindDatabaseWithName,
    FindEventBusWithName,
    FindIdentityServiceWithName,
    FindServerWithName,
    FindAllFilesWithExtensionInDirectory)

from modules.templating.templating import (replace_template_file,
    filter_region,
    filter_region_with_tag,
    filter_sub_region,
    clear_file_region_tags,
    ClearRegionLines,
    Clear_File_Region_Marks)

from modules.databases.postgre import BuildPostgreConnectionString
from modules.databases.mysql import BuildMysqlConnectionString

from modules.templating.csharp import Csharp

import os
import shutil



class IdentityServer4(BaseModule):
    
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        self.csharp_templater = Csharp(projectOptions,project_templates_paths,outputPath)
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def FindApiServicesUsesIs4(self, i_service_name):
        api_services = []
        if 'api_services' not in self.projectOptions:
            return api_services
        if len(self.projectOptions['api_services']) < 1:
            return api_services
        for service in self.projectOptions['api_services']:
            for _, value in service.items():            
                if 'authorization' in value:
                    if value['authorization']['issuer'] == i_service_name:
                        api_services.append(value)
        return api_services
    def FindClientsUsesIs4(self, i_service_name):
        clients = []
        if 'clients' not in self.projectOptions:
            return clients
        if len(self.projectOptions['clients']) < 1:
            return clients
        for client in self.projectOptions['clients']:
            for _, value in client.items():            
                if 'authorization' in value:
                    if value['authorization']['issuer'] == i_service_name:
                        clients.append(value)
        return clients

    def HandleIs4ClientConfiguration(self, clients, identity_service, is4_copy_folder):
        clients_txt_template_file = os.path.join(
            is4_copy_folder,
            'src',
            'Host',
            'Configuration',
            'client.config.txt'
            )
        clients_cs_template_file = os.path.join(
            is4_copy_folder,
            'src',
            'Host',
            'Configuration',
            'Clients.cs'
            )
        with open(clients_txt_template_file) as temp_file:
            template_string = temp_file.read()
        client_config_as_cs = ""
        client_count = len(clients)
        for client_ind, client in enumerate(clients):
            client_host = 'http://'+client['name'].lower()+'.localhost'
            redirect_url_templ_val = ( InDbQ(client_host) +',\n' 
            + '\t\t\t\t\t\t'+ InDbQ(client_host+'/silent-renew.html') +',\n' 
            + '\t\t\t\t\t\t'+ InDbQ(client_host+'/login-callback.html')) 
            
            post_logout_redirect_url_val = ( InDbQ(client_host) +',\n'
            + '\t\t\t\t\t\t'+ InDbQ(client_host+'/loggedout'))

            cors_origins_val = InDbQ(client_host) +',\n'

            grant_type_val = 'GrantTypes.Implicit'
            if client['type'].startswith('angular'):
                grant_type_val = 'GrantTypes.Implicit'
            elif client['type'].startswith('native'):
                grant_type_val = 'GrantTypes.ResourceOwnerPassword'
            elif client['type'].startswith('mobile'):
                grant_type_val = 'GrantTypes.ResourceOwnerPassword'
            client_config_as_cs += (
                template_string 
                .replace('{{client:id}}',client['name']) 
                .replace('{{client:name}}',client['name']) 
                .replace('{{client:url}}',client_host) 
                .replace('{{client:accesstokentype}}','AccessTokenType.Reference') 
                .replace('{{client:redirecturls}}',redirect_url_templ_val) 
                .replace('{{client:logoutredirecturls}}',post_logout_redirect_url_val)
                .replace('{{client:corsorigins}}', cors_origins_val)
                .replace('{{client:granttype}}',grant_type_val)
                )
            
            if 'scopes' in client['authorization']:
                scope_val = ''
                scope_options = client['authorization']['scopes']
                scope_count = len(scope_options)
                for scope_index, scope in enumerate(scope_options):
                    if(scope_index != 0 ):
                        scope_val  += '\t\t\t\t\t\t'
                    scope_val += InDbQ(scope)
                    if (scope_count-1 != scope_index):
                        scope_val += ','
                    scope_val += '\n'
                client_config_as_cs = client_config_as_cs.replace('{{client:scopes}}',scope_val)
            client_config_as_cs += ',\n'
            # Dev configuration
            client_host = 'http://localhost:'+str(client['port'])
            redirect_url_templ_val = ( InDbQ(client_host) +',\n' 
            + '\t\t\t\t\t\t'+ InDbQ(client_host+'/silent-renew.html') +',\n' 
            + '\t\t\t\t\t\t'+ InDbQ(client_host+'/login-callback.html')) 
            
            post_logout_redirect_url_val = ( InDbQ(client_host) +',\n'
            + '\t\t\t\t\t\t'+ InDbQ(client_host+'/loggedout'))

            cors_origins_val = InDbQ(client_host) +',\n'

            grant_type_val = 'GrantTypes.Implicit'
            if client['type'].startswith('angular'):
                grant_type_val = 'GrantTypes.Implicit'
            elif client['type'].startswith('native'):
                grant_type_val = 'GrantTypes.ResourceOwnerPassword'
            elif client['type'].startswith('mobile'):
                grant_type_val = 'GrantTypes.ResourceOwnerPassword'
            client_config_as_cs += (
                template_string 
                .replace('{{client:id}}',client['name']+'dev') 
                .replace('{{client:name}}',client['name']+'dev') 
                .replace('{{client:url}}',client_host) 
                .replace('{{client:accesstokentype}}','AccessTokenType.Reference') 
                .replace('{{client:redirecturls}}',redirect_url_templ_val) 
                .replace('{{client:logoutredirecturls}}',post_logout_redirect_url_val)
                .replace('{{client:corsorigins}}', cors_origins_val)
                .replace('{{client:granttype}}',grant_type_val)
                )
            
            if 'scopes' in client['authorization']:
                scope_val = ''
                scope_options = client['authorization']['scopes']
                scope_count = len(scope_options)
                for scope_index, scope in enumerate(scope_options):
                    if(scope_index != 0 ):
                        scope_val  += '\t\t\t\t\t\t'
                    scope_val += InDbQ(scope)
                    if (scope_count-1 != scope_index):
                        scope_val += ','
                    scope_val += '\n'
                client_config_as_cs = client_config_as_cs.replace('{{client:scopes}}',scope_val)
            if (client_ind != client_count-1):
                client_config_as_cs += ',\n'

        with open(clients_cs_template_file,'r') as cs_file:
            cs_content = cs_file.read()
        os.remove(clients_cs_template_file)
        os.remove(clients_txt_template_file)
        cs_content = cs_content.replace('//& replace (clients)', client_config_as_cs)
        with open(clients_cs_template_file,'w') as cs_file_new:
            cs_file_new.write(cs_content)
            
    def HandleIs4ResourcesConfiguration(self, resources, identity_service, is4_copy_folder):
        resources_txt_template_file = os.path.join(
            is4_copy_folder,
            'src',
            'Host',
            'Configuration',
            'resource.config.txt'
            )
        resources_cs_template_file = os.path.join(
            is4_copy_folder,
            'src',
            'Host',
            'Configuration',
            'Resources.cs'
            )
        with open(resources_txt_template_file) as temp_file:
            template_string = temp_file.read()
        resource_config_as_cs = ""
        resource_count = len(resources)
        for resource_ind, resource in enumerate(resources):
            resource_host = resource['name'].lower()+'.localhost' 
            resource_config_as_cs += (
                template_string 
                .replace('{{resource:name}}',resource['name'])
                .replace('{{resource:displayname}}',resource['name'])
                )
            
            if 'secrets' in resource['authorization']:
                secret_val = ''
                secrets = resource['authorization']['secrets']
                secret_count = len(secrets)            
                for secret_index, secret in enumerate(secrets):
                    if(secret_index != 0 ):
                        secret_val  += '\t\t\t\t\t\t'
                    secret_val +='new Secret('+InDbQ(secret)+'.Sha256())'
                    if (secret_count-1 != secret_index):
                        secret_val += ','
                    secret_val += '\n'
                resource_config_as_cs = resource_config_as_cs.replace('{{resource:secrets}}',secret_val)
            if 'avaliable_scopes' in resource['authorization']:
                scope_val = ''
                scope_options = resource['authorization']['avaliable_scopes']
                scope_count = len(scope_options)
                for scope_index, scope in enumerate(scope_options):
                    if(scope_index != 0 ):
                        scope_val  += '\t\t\t\t\t\t'
                    scope_val += 'new Scope() { Name='+InDbQ(scope)+'}'
                    if (scope_count-1 != scope_index):
                        scope_val += ','
                    scope_val += '\n'
                resource_config_as_cs = resource_config_as_cs.replace('{{resource:scopes}}',scope_val)
            else: 
                resource_config_as_cs = resource_config_as_cs.replace('{{resource:scopes}}','')
            if 'user_claims' in resource['authorization']:
                claims_val = ''
                claims_options = resource['authorization']['user_claims']
                claims_count = len(claims_options)
                for claims_index, claim in enumerate(claims_options):
                    if(claims_index != 0 ):
                        claims_val  += '\t\t\t\t\t\t'
                    claims_val += InDbQ(claim)
                    if (claims_count-1 != claims_index):
                        claims_val += ','
                    claims_val += '\n'
                resource_config_as_cs = resource_config_as_cs.replace('{{resource:userclaims}}',claims_val)
            else:
                resource_config_as_cs = resource_config_as_cs.replace('{{resource:userclaims}}','JwtClaimTypes.Scope')
            if (resource_ind != resource_count-1):
                resource_config_as_cs += ',\n'

        with open(resources_cs_template_file,'r') as cs_file:
            cs_content = cs_file.read()
        os.remove(resources_cs_template_file)
        os.remove(resources_txt_template_file)
        cs_content = cs_content.replace('//& replace (apiresources)', resource_config_as_cs)
        with open(resources_cs_template_file,'w') as cs_file_new:
            cs_file_new.write(cs_content)

    def HanldeIs4Csproj(self, is4_options,is4_folder):
        print ('Configuring Identity Server 4 csproj')
        host_csproj_path = os.path.join(is4_folder,
            'src',
            'Host',
            'Host.csproj')
        host_eflib_csproj_path = os.path.join(is4_folder,
            'src',
            'IdentityServer4.EntityFramework',
            'IdentityServer4.EntityFramework.csproj')
        # Handle Host Application
        self.csharp_templater.HandleCsprojLogging(is4_options,host_csproj_path)
        self.csharp_templater.HandleCsprojDatabase(is4_options,host_csproj_path)
        self.csharp_templater.HandleCsprojEventbus(is4_options,host_csproj_path)

        self.csharp_templater.HandleCsprojLogging(is4_options,host_eflib_csproj_path)
        self.csharp_templater.HandleCsprojDatabase(is4_options,host_eflib_csproj_path)
        self.csharp_templater.HandleCsprojEventbus(is4_options,host_eflib_csproj_path)

    def BuildConnStringForIs4(self, identity_options):
        database_instance_name = identity_options['database']['provider']
        database_instance = FindDatabaseWithName(self.projectOptions, database_instance_name)
        database_type = database_instance['type']
        user_connection_string ='' 
        config_connection_string = ''
        user = 'doom'
        password = 'machine'
        if database_type=='mysql' or database_type=='postgresql':
            if 'username' in database_instance:
                user = database_instance['username']
            if 'password' in database_instance:
                password = database_instance['password'] 
        if database_type == 'mysql':            
            user_connection_string, user_connection_string_dev = BuildPostgreConnectionString(database_instance['name'],identity_options['name'].lower()+'_users',user,password)        
            config_connection_string, config_connection_string_dev = BuildPostgreConnectionString(database_instance['name'],identity_options['name'].lower()+'_config',user,password)
        elif database_type == 'postgresql':
            user_connection_string, user_connection_string_dev = BuildPostgreConnectionString(database_instance['name'],identity_options['name'].lower()+'_users',user,password)        
            config_connection_string, config_connection_string_dev = BuildPostgreConnectionString(database_instance['name'],identity_options['name'].lower()+'_config',user,password)
        conn_strings = {}
        conn_strings['user_connection_string'] = user_connection_string
        conn_strings['user_connection_string_dev'] = user_connection_string_dev
        conn_strings['config_connection_string'] = config_connection_string
        conn_strings['config_connection_string_dev'] = config_connection_string_dev
        return conn_strings
    def HandleConnectionStringForIs4(self, identity_options ,is4_copy_folder):
        conn_strings =  self.BuildConnStringForIs4(identity_options)
        startup_file_path = os.path.join(is4_copy_folder,'src','Host','Startup.cs')
        replace_dict = {
            '{{database:usersconnectionstring-dev}}':conn_strings['user_connection_string_dev'],
            '{{database:usersconnectionstring}}': conn_strings['user_connection_string'],
            '{{database:configsConnectionString-dev}}': conn_strings['config_connection_string_dev'],
            '{{database:configsConnectionString}}': conn_strings['config_connection_string']
        }
        replace_template_file(startup_file_path,replace_dict)
    def HandleEventBusForIs4(self, i_srv, is4_copy_folder):
        eventbus_srv = FindEventBusWithName(self.projectOptions, i_srv['eventbus']['provider'])
        startup_file_path = os.path.join(is4_copy_folder,'src','Host','Startup.cs')
        repleceDict = {
            '{{rabbitmq:host}}': 'rabbitmq://'+eventbus_srv['name'],
            '{{rabbitmq:host-dev}}' : 'rabbitmq://localhost'
        }
        if 'docker_compose_override' in eventbus_srv:
            if 'environment' in eventbus_srv['docker_compose_override']:
                if 'RABBITMQ_DEFAULT_USER' in eventbus_srv['docker_compose_override']['environment']:
                    repleceDict['{{rabbitmq:user:username}}'] = eventbus_srv['docker_compose_override']['environment']['RABBITMQ_DEFAULT_USER']
                else:
                    repleceDict['{{rabbitmq:user:username}}'] = 'doom'
                if 'RABBITMQ_DEFAULT_PASSWORD' in eventbus_srv['docker_compose_override']['environment']:
                    repleceDict['{{rabbitmq:user:password}}'] = eventbus_srv['docker_compose_override']['environment']['RABBITMQ_DEFAULT_PASSWORD']
                else:
                    repleceDict['{{rabbitmq:user:password}}'] = 'machine'
            else:
                repleceDict['{{rabbitmq:user:username}}'] = 'doom'
                repleceDict['{{rabbitmq:user:password}}'] = 'machine'
        else:
            repleceDict['{{rabbitmq:user:username}}'] = 'doom'
            repleceDict['{{rabbitmq:user:password}}'] = 'machine'
        replace_template_file(startup_file_path, repleceDict)

    def HandleIs4DockerFile(self, identity_service, is4_copy_folder):
        docker_file_path = os.path.join(is4_copy_folder,'Dockerfile')
        docker_replace_dict = {}
        docker_replace_dict['{{port}}'] = str(identity_service['port'])
        replace_template_file(docker_file_path,docker_replace_dict)
    def HandleStartupForIs4(self, identity_service, is4_copy_folder):
        startup_file_path = os.path.join(is4_copy_folder,'src','Host','Startup.cs')
        program_file_path = os.path.join(is4_copy_folder,'src','Host','Program.cs')
        self.csharp_templater.HandleCSharpEventbus(identity_service,startup_file_path)
        self.csharp_templater.HandleCSharpDatabase(identity_service,startup_file_path)
        self.csharp_templater.HandleCSharpLogging(identity_service,startup_file_path)
        self.csharp_templater.HandleCSharpServer(identity_service,startup_file_path)
        self.csharp_templater.HandleCSharpLogging(identity_service,program_file_path)
    def GetIdentityServerDockerOptions(self, identity_service):
        is4_docker_props = {
            'image': identity_service['name'].lower(),
            'build': {
                'context': 'src/IdentityServices/'+identity_service['name']+'/',
                'dockerfile': 'Dockerfile'
            },
            'restart': 'on-failure',
            'ports': [],
            'links': [],
            'depends_on':[],
            'networks':['localnet'],        
        }
        is4_docker_props['links'].append(identity_service['database']['provider'])
        is4_docker_props['depends_on'].append(identity_service['database']['provider'])
        if 'port' in identity_service:        
            is4_docker_props['ports'].append(str(identity_service['port'])+':'+str(identity_service['port']))
        eventbus_enabled = 'eventbus' in identity_service
        if eventbus_enabled:
            eb_provider = identity_service['eventbus']['provider']        
            is4_docker_props['links'].append(eb_provider)
            is4_docker_props['depends_on'].append(eb_provider)

        return  is4_docker_props
    def HandleIs4Cleaning(self, copy_folder):
        src_path = os.path.join(copy_folder)
        file_clean_paths = FindAllFilesWithExtensionInDirectory(src_path,('.cs','.csproj'))
        ClearRegionLines(file_clean_paths)
    def HandleIdentityServer4(self, identity_service):
        
        docker_options = self.GetIdentityServerDockerOptions(identity_service)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(identity_service['name'],docker_options)

        identityServicesPath = os.path.join(self.project_templates_paths,'identity_services')
        is4_template_folder = os.path.join(identityServicesPath,'identityserver4ef')
        srcDir = os.path.join(self.outputPath,'src')
        is4_copy_folder = os.path.join(srcDir,'IdentityServices',identity_service['name'])
        if os.path.isdir(is4_copy_folder):
            shutil.rmtree(is4_copy_folder,ignore_errors=True)  
        # TODO: Swap shutil operations
        #shutil.copytree(is4_template_folder,is4_copy_folder,ignore=shutil.ignore_patterns('bin*','obj*'))
        shutil.copytree(is4_template_folder,is4_copy_folder)
        
        api_services_using_is4 = self.FindApiServicesUsesIs4(identity_service['name'])
        clients_using_is4 = self.FindClientsUsesIs4(identity_service['name'])

        self.HandleIs4ClientConfiguration(clients_using_is4,identity_service,is4_copy_folder)
        self.HandleIs4ResourcesConfiguration(api_services_using_is4,identity_service,is4_copy_folder)
        self.HanldeIs4Csproj(identity_service,is4_copy_folder)
        self.HandleConnectionStringForIs4(identity_service ,is4_copy_folder)
        if 'eventbus' in identity_service:
            self.HandleEventBusForIs4(identity_service, is4_copy_folder)
        self.HandleIs4DockerFile(identity_service, is4_copy_folder)
        self.HandleStartupForIs4(identity_service, is4_copy_folder)
        self.HandleIs4Cleaning(is4_copy_folder)
        
    


#!/usr/bin/env python
#pylint: disable-msg=W0612

import readline
import argparse
import yaml
import os
import sys

from modules.databases.mongodb import MongoDb
from modules.databases.mysql import Mysql
from modules.databases.postgre import Postgre
from modules.databases.redis import Redis

from modules.clients.angular import Angular

from modules.eventbus.rabbitmq import RabbitMq

from modules.identity.identityserver4 import IdentityServer4

from modules.services.dotnetapi import DotnetApi
from modules.services.nodeapi import NodeApi

from modules.servers.nginx import Nginx
from modules.devops.docker import Docker

parser = argparse.ArgumentParser(description='Generate and start your new project from moon')
parser.add_argument('--config','-c', type=str, help='Configuration file', required=True)
parser.add_argument('--output','-o', type=str, help='Output directory', required=False)
parser.add_argument('--force','-f',type=bool)

args = vars(parser.parse_args())
configFilePath = os.path.join(os.getcwd(),args['config'])
print (args['output'])

projectOptions = {}

scriptPath = os.path.dirname(os.path.realpath(sys.argv[0]))
templatesPath = os.path.normpath(os.path.join(scriptPath,'templatefiles'))

# project_templates_paths = {    
# 
#     'serversPath' : os.path.join(templatesPath,'servers'),
#     'apiServicesPath' : os.path.join(templatesPath,'api_services'),
#     'clientsPath' : os.path.join(templatesPath,'clients'),
#     'databasesPath' : os.path.join(templatesPath,'databases'),
#     'eventbusPath' : os.path.join(templatesPath,'eventbus'),
#     'identityServicesPath' : os.path.join(templatesPath,'identity_services')
# 
# }


def CreateProjectDirectory(projectName):

    project_output_dir = None
    if args['output'] is not None :
        project_output_dir = os.path.normpath(os.path.join(os.getcwd(),args['output']))
    else:
        project_output_dir = os.path.normpath(os.path.join(os.getcwd(),projectName))

    srcDir = os.path.normpath(os.path.join(project_output_dir,"src"))
    docker_volume_dir = os.path.normpath(os.path.join(project_output_dir,"docker_volumes"))
    if not os.path.isdir(srcDir):
        os.makedirs(srcDir)
    if not os.path.isdir(docker_volume_dir):
        os.makedirs(docker_volume_dir)
    # Create README.md
    f = open(os.path.normpath(os.path.join(project_output_dir,'README.md')), 'w+')
    f.write('#'+projectName)
    f.close()
    return project_output_dir, srcDir
    

def HandleServers(servers):
    print ('Configuring Servers')
    for server in servers:
        server_options = list(server.values())[0]
        print('Building'+ server_options['name'])
        if server_options['type'] == 'nginx':
            nginx_instance = Nginx(projectOptions,templatesPath,project_output_dir, server_options)
            nginx_instance.GenerateNginxInstance()


def HandleDatabases(databases):
    print ('Configuring Databases')
    for db in databases:
        db_options = list(db.values())[0]
        print('Scaffolding '+db_options['name'])
        if(db_options['type'] == 'postgresql'):
            postgre_ins = Postgre(projectOptions,templatesPath,project_output_dir)
            postgre_ins.HandlePostgre(db_options)
        if(db_options['type'] == 'mysql'):
            mysql_ins = Mysql(projectOptions,templatesPath,project_output_dir)
            mysql_ins.HandleMysql(db_options)
        elif db_options['type'] == 'redis':
            redis_ins = Redis(projectOptions,templatesPath,project_output_dir)
            redis_ins.HandleRedisDatabase(db_options)
        elif db_options['type'] == 'mongodb':
            mongo_ins = MongoDb(projectOptions,templatesPath,project_output_dir)
            mongo_ins.HandleMongoDb(db_options)


def HandleEventBus(eventbuses):
    print ('Configuring Bus Instances..')
    for evenbus in eventbuses:
        evenbus_options = list(evenbus.values())[0]
        print('Scaffolding '+evenbus_options['name'])
        if(evenbus_options['type'] == 'rabbitmq'):
            rabbit_ins = RabbitMq(projectOptions,templatesPath,project_output_dir)
            rabbit_ins.HandleRabbitMq(evenbus_options)

def HandleIdentityServices(identity_services):
    print ('Scaffolding Identity Services...')
    for i_service in identity_services:
        i_service_options = list(i_service.values())[0]
        if (i_service_options['type']=='identityserver4'):
            is4_ins = IdentityServer4(projectOptions,templatesPath,project_output_dir)
            is4_ins.HandleIdentityServer4(i_service_options)


def HandleApiServices(api_services):
    print ('Scaffolding Api Services')
    for api_service in api_services:
        api_service_options = list(api_service.values())[0]
        if(api_service_options['type']=='dotnet_web_api'):
            dotnetapi_ins = DotnetApi(projectOptions,templatesPath,project_output_dir)
            dotnetapi_ins.HandleDotnetApiService(api_service_options)
        if(api_service_options['type']=='node_web_api'):
            nodeapi_ins = NodeApi(projectOptions,templatesPath,project_output_dir)
            nodeapi_ins.HandleNodeWebApi(api_service_options)



def HandleClients(clients):
    print ('Scaffolding Clients')
    for client in clients:
        client_options = list(client.values())[0]
        if(client_options['type']=='angular_cli_6_ssr'):
            angular_ins = Angular(projectOptions,templatesPath,project_output_dir)
            angular_ins.HandleAngular6SsrClient(client_options) 



with open(configFilePath, 'r') as stream:
    try:
        # Load Yaml
        projectOptions = yaml.load(stream)
        if not ('name' in projectOptions):
            print('Please Provide a valid project name')
            exit
        projectName = projectOptions['name']
        # Create Project Files
        project_output_dir, srcDir = CreateProjectDirectory(projectName)
        # init as singleton
        dock = Docker(projectOptions,templatesPath,project_output_dir)

        # Create Servers (Nginx Apache)
        if('servers' in projectOptions):
            HandleServers(projectOptions['servers'])

        # Create Databases (Postgre, Mysql, Redis)
        if('databases' in projectOptions):
            HandleDatabases(projectOptions['databases'])
            
        # Configure Eventbus Instances (Rabbitmq)
        if('eventbus' in projectOptions):
            HandleEventBus(projectOptions['eventbus'])

        # Create and configure identity_services
        if('identity_services' in projectOptions):
            HandleIdentityServices(projectOptions['identity_services'])

        # Create and configure api_serviecs
        if('api_services' in projectOptions):
            HandleApiServices(projectOptions['api_services'])
        # Create and configure clients
        if('clients' in projectOptions):
            HandleClients(projectOptions['clients'])

        docker_compose_path = os.path.join(project_output_dir,'docker-compose.yml')
        dock.DumpTo(docker_compose_path)
        dock.DockerComposeFinalization(docker_compose_path)

        
    except yaml.YAMLError as exc:
        print('Error parsing yml document')
        print(exc)    




from modules.servers.server import Server
from modules.devops.docker import Docker
import shutil
import os
import nginx


class Nginx(Server):
    def __init__(self, projectOptions, project_templates_paths, outputPath , server_options):
        self.server_options = server_options
        super().__init__(projectOptions,project_templates_paths,outputPath, server_options)

    def GenerateNginxInstance(self):
        serversPath = os.path.join(self.project_templates_paths,'servers')
        nginxTemplateFolder = os.path.join(serversPath,'nginx')
        folderPath = os.path.normpath(os.path.join(self.outputPath, self.server_options['name']))
        nginxPath = os.path.join(folderPath,'nginx.conf')
        if os.path.isdir(folderPath):
            shutil.rmtree(folderPath, ignore_errors=True)
        shutil.copytree(nginxTemplateFolder,folderPath)
        api_services_uses_nginx = self.FindApiServicesUsesNginx(self.server_options['name'])
        clients_uses_nginx = self.FindClientsUsesNginx(self.server_options['name'])
        identity_uses_nginx = self.FindIdentityServicesUsesNginx(self.server_options['name'])
        nginxConfig = self.BuildNginxConfiguration(self.server_options,api_services_uses_nginx, clients_uses_nginx,identity_uses_nginx)
        docker_config = self.BuildNginxDockerOptions(api_services_uses_nginx, clients_uses_nginx,identity_uses_nginx)
        
        docker_instance = Docker.getInstance()
        docker_instance.AddService(self.server_options['name'],docker_config)
        nginx.dumpf(nginxConfig, nginxPath)
        
    def BuildNginxConfiguration(self, server,api_services,clients,identity_services):
        print ("Configuring Nginx Server")
        serverConfig = server['config']
        config = nginx.Conf()
        # Add Root Configurations
        for key,value in serverConfig.items():
            if (not isinstance(value,dict)):
                config.add(nginx.Key(key,value))
        events = nginx.Events()
        httpConf = nginx.Http()
        # Add Event Configurations
        if ('events' in serverConfig):
            for key, value in serverConfig['events'].items():        
                events.add(nginx.Key(key,value))
        config.add(events)
        # Add Http Configurations
        if ('http' in serverConfig):
            for key,value in serverConfig['http'].items():
                httpConf.add(nginx.Key(key,value))  
        # Add Services To Http
        for api_service in api_services:
            nginxServer = nginx.Server(
                nginx.Key('listen', '80'),
                nginx.Key('server_name', str.lower(api_service['name'])+'.localhost'),
            )
            proxy_pass = 'http://'+api_service['name']+':'+str(api_service['port'])+'/'
            location = nginx.Location(
                '/',
                nginx.Key('proxy_http_version', '1.1'),
                nginx.Key('proxy_set_header', 'Upgrade $http_upgrade'),
                nginx.Key('proxy_set_header', 'Connection keep-alive'),
                nginx.Key('proxy_set_header', 'X-Forwarded-For $proxy_add_x_forwarded_for'),
                nginx.Key('proxy_set_header', 'Host $host'),
                nginx.Key('proxy_set_header', 'X-NginX-Proxy true'),
                nginx.Key('proxy_pass', proxy_pass)
            )
            nginxServer.add(location)
            httpConf.add(nginxServer)
        for i_service in identity_services:
            nginxServer = nginx.Server(
                nginx.Key('listen', '80'),
                nginx.Key('server_name', str.lower(i_service['name'])+'.localhost'),
            )
            #pylint: disable-msg=E1121
            proxy_pass = 'http://'+i_service['name']+':'+str(i_service['port'])+'/'
            location = nginx.Location(
                '/',
                nginx.Key('proxy_http_version', '1.1'),
                nginx.Key('proxy_set_header', 'Upgrade $http_upgrade'),
                nginx.Key('proxy_set_header', 'Connection keep-alive'),
                nginx.Key('proxy_set_header', 'X-Forwarded-For $proxy_add_x_forwarded_for'),
                nginx.Key('proxy_set_header', 'Host $host'),
                nginx.Key('proxy_set_header', 'X-NginX-Proxy true'),
                nginx.Key('proxy_pass', proxy_pass)
            )
            nginxServer.add(location)
            httpConf.add(nginxServer)
        for client in clients:
            nginxServer = nginx.Server(
                nginx.Key('listen', '80'),
                nginx.Key('server_name', str.lower(client['name'])+'.localhost'),
            )
            #pylint: disable-msg=E1121
            proxy_pass = 'http://'+client['name']+':'+str(client['port'])+'/'
            location = nginx.Location(
                '/',
                nginx.Key('proxy_http_version', '1.1'),
                nginx.Key('proxy_set_header', 'Upgrade $http_upgrade'),
                nginx.Key('proxy_set_header', 'Connection keep-alive'),
                nginx.Key('proxy_set_header', 'X-Forwarded-For $proxy_add_x_forwarded_for'),
                nginx.Key('proxy_set_header', 'Host $host'),
                nginx.Key('proxy_set_header', 'X-NginX-Proxy true'),
                nginx.Key('proxy_pass',proxy_pass)
            )
            nginxServer.add(location)
            httpConf.add(nginxServer)
        config.add(httpConf)
        return config
        
    def BuildNginxDockerOptions(self,api_services, clients,identity_services):
        nginxOptions = {        
                'image': 'nginxhttp',
                'container_name': self.server_options['name'].lower(),
                'ports': [],
                'links': [],
                'restart': 'on-failure',
                'depends_on':[],
                'networks': ['localnet'],
                'build': {'context': self.server_options['name']+'/', 'dockerfile':'Dockerfile'}        
        }
        if 'ports' in self.server_options:        
            for port in self.server_options['ports']:
                nginxOptions['ports'].append(str(port)+':'+str(port))
        else: 
            nginxOptions['ports'].append("80:80")
            nginxOptions['ports'].append("443:443")
        for service in api_services:
            nginxOptions['links'].append(service['name'])
            nginxOptions['depends_on'].append(service['name'])
        for client in clients:
            nginxOptions['links'].append(client['name'])
            nginxOptions['depends_on'].append(client['name'])
        for i_service in identity_services:
            nginxOptions['links'].append(i_service['name'])
            nginxOptions['depends_on'].append(i_service['name'])
        
        return nginxOptions
    def FindApiServicesUsesNginx(self,serverName):
        services = []
        for service in self.projectOptions['api_services']:
            for _, value in service.items():
                if 'server' in value:
                    if value['server']['provider'] == serverName:
                        services.append(value)
        return services
    def FindClientsUsesNginx(self,serverName):
        clients = []
        for client in self.projectOptions['clients']:
            for _, value in client.items():            
                if 'server' in value:
                    if value['server']['provider'] == serverName:
                        clients.append(value)
        return clients
    def FindIdentityServicesUsesNginx(self,serverName):
        i_services = []
        for i_service in self.projectOptions['identity_services']:
            for _, value in i_service.items():            
                if 'server' in value:
                    if value['server']['provider'] == serverName:
                        i_services.append(value)
        return i_services



from modules.basemodule import BaseModule
from modules.devops.docker import Docker
from modules.databases.redis import BuildRedisConnectionString
from modules.utils.utils import (InDbQ,to_camelcase,
    FindClientWithName,
    FindDatabaseWithName,
    FindEventBusWithName,
    FindIdentityServiceWithName,
    FindServerWithName,
    FindAllFilesWithExtensionInDirectory)

from modules.templating.templating import (replace_template_file,
    filter_region,
    filter_region_with_tag,
    filter_sub_region,
    clear_file_region_tags,
    ClearRegionLines,
    Clear_File_Region_Marks)

from modules.databases.postgre import BuildPostgreConnectionString
from modules.databases.mysql import BuildMysqlConnectionString

from modules.templating.csharp import Csharp

import os
import shutil


class DotnetApi(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        self.csharp_templater = Csharp(projectOptions,project_templates_paths,outputPath)
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def HandleDotnetApiService(self, api_service_options):
        CamelCaseName = to_camelcase(api_service_options['name'])
        apiServicesPath = os.path.join(self.project_templates_paths,'api_services')
        api_template_folder = os.path.join(apiServicesPath,'dotnet_web_api','src')
        srcDir = os.path.join(self.outputPath,'src')
        api_copy_folder = os.path.join(srcDir,'ApiServices',CamelCaseName )
        if os.path.isdir(api_copy_folder):
            shutil.rmtree(api_copy_folder,ignore_errors=True)
        # TODO: Swap shutil operations
        #shutil.copytree(api_template_folder,api_copy_folder,ignore=shutil.ignore_patterns('bin*','obj*'))
        shutil.copytree(api_template_folder,api_copy_folder)
        api_src_folder = os.path.join(srcDir,'ApiServices',CamelCaseName,'DotnetWebApi')
        api_src_rename_folder = os.path.join(srcDir,'ApiServices',CamelCaseName,'src')
        api_csproj_folder = os.path.join(srcDir,'ApiServices',CamelCaseName,'src','DotnetWebApi.csproj')
        api_csproj_rename_folder = os.path.join(srcDir,'ApiServices',CamelCaseName,'src',CamelCaseName+'.csproj')
        
        if not os.path.isdir(api_src_rename_folder):
            shutil.copytree(api_src_folder,api_src_rename_folder)
            shutil.rmtree( api_src_folder,ignore_errors=True)
        else: 
            shutil.rmtree( api_src_rename_folder,ignore_errors=True)
            shutil.copytree(api_src_folder,api_src_rename_folder)

        if not os.path.exists(api_csproj_rename_folder):
            shutil.copy(api_csproj_folder,api_csproj_rename_folder)
            os.remove( api_csproj_folder)
        else: 
            os.remove(api_csproj_rename_folder)
            shutil.copy(api_csproj_folder,api_csproj_rename_folder)


        self.HandleDotnetApiCsproj(api_service_options,api_copy_folder)
        self.HandleDotnetApiStartup(api_service_options,api_copy_folder)
        self.HandleDotnetApiProgramFile(api_service_options,api_copy_folder)
        self.HandleDotnetApiDbContext(api_service_options,api_copy_folder)
        self.HandleDotnetApiNameSpaceAndCleaning(api_service_options,api_copy_folder)
        self.HandleDotnetApiDockerFile(api_service_options,api_copy_folder)

        docker_config = self.HandleDotnetApiDockerCompose(api_service_options,api_copy_folder)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(api_service_options['name'], docker_config)

    def HandleDotnetApiCsproj(self, dotnet_service, api_copy_folder):
        print ('Handle DotnetApi Csproj File')
        api_csproj_path = os.path.join(api_copy_folder,
            'src',
            to_camelcase(dotnet_service['name'])+'.csproj')
        # Handle Host Application
        self.csharp_templater.HandleCsprojLogging(dotnet_service,api_csproj_path)
        self.csharp_templater.HandleCsprojDatabase(dotnet_service,api_csproj_path)
        self.csharp_templater.HandleCsprojEventbus(dotnet_service,api_csproj_path)
    def BuildConnStringForDotnetApi(self, dotnet_options):
        database_instance_name = dotnet_options['database']['provider']
        database_instance = FindDatabaseWithName(self.projectOptions, database_instance_name)
        database_type = database_instance['type']
        connection_string ='' 
        user = 'doom'
        password = 'machine'
        if database_type=='mysql' or database_type=='postgresql':
            if 'username' in database_instance:
                user = database_instance['username']
            if 'password' in database_instance:
                password = database_instance['password']
        if database_type == 'mysql':            
            connection_string, connection_string_dev = BuildMysqlConnectionString(database_instance['name'],dotnet_options['name'],user,password)        
        elif database_type == 'postgresql':
            connection_string, connection_string_dev = BuildPostgreConnectionString(database_instance['name'],dotnet_options['name'],user,password)        
                
        return connection_string , connection_string_dev

    def HandleDotnetApiStartup(self, dotnet_service, api_copy_folder):
        print ('Handle DotnetApi Startup.cs File')
        api_startup_path = os.path.join(api_copy_folder,
            'src',
            'Startup.cs')
        
        self.csharp_templater.HandleCSharpDatabase(dotnet_service,api_startup_path)
        self.csharp_templater.HandleCSharpCache(dotnet_service,api_startup_path)
        self.csharp_templater.HandleCSharpEventbus(dotnet_service,api_startup_path)
        self.csharp_templater.HandleCSharpLogging(dotnet_service,api_startup_path)
        self.csharp_templater.HandleCSharpServer(dotnet_service,api_startup_path)
        self.csharp_templater.HandleCSharpSwagger(dotnet_service,api_startup_path)
        # Set DBContext Name
        CamelCaseName = to_camelcase(dotnet_service['name'])
        replaceDict = {
            'NameContext': CamelCaseName.replace('.','') + 'Context'    
        }
        if 'database' in dotnet_service:
            
            conn_string, conn_string_dev = self.BuildConnStringForDotnetApi(dotnet_service)
            replaceDict['{{database:connectionString}}'] = conn_string
            replaceDict['{{database:connectionString-dev}}'] = conn_string_dev

        if 'cache' in dotnet_service:
            if dotnet_service['cache']['type'] == 'redis':
                redis_instance = FindDatabaseWithName(self.projectOptions, dotnet_service['cache']['redis_options']['redis_server'])
                if redis_instance is None:
                    print ('Warning: Redis instance could not found. Configuration left default')
                else:
                    redis_conn_string, redis_conn_string_dev = BuildRedisConnectionString(redis_instance)
                    replaceDict['{{redis_options:connection}}'] = redis_conn_string
                    replaceDict['{{redis_options:connection-dev}}'] =  redis_conn_string_dev
                    if 'redis_instance_name' in dotnet_service['cache']['redis_options']:
                        replaceDict['{{redis_options:instance_name}}'] = dotnet_service['cache']['redis_options']['redis_instance_name']
        if 'authorization' in dotnet_service:
            issuer = dotnet_service['authorization']['issuer']
            if issuer is None:
                print ('Error: Identity Issuer for '+dotnet_service['name']+' is required')
            identity_instance = FindIdentityServiceWithName(self.projectOptions, issuer)
            if identity_instance is None:
                print ('Error: Identity Service Instance for '+dotnet_service['name']+' could not found')
            else:
                replaceDict['{{authorization:api_name}}'] = dotnet_service['name']        
                replaceDict['{{authorization:authority}}'] = str.lower(identity_instance['name'])+'.localhost'
                replaceDict['{{authorization:authority-dev}}'] = 'http://localhost:'+str(identity_instance['port'])
                if 'api_secret' in dotnet_service['authorization']:
                    replaceDict['{{authorization:api_secret}}'] = dotnet_service['authorization']['secrets'][0]
                else:
                    # Set Default Secret
                    replaceDict['{{authorization:api_secret}}'] = 'secret'

        replace_template_file(api_startup_path,replaceDict)

    def HandleDotnetApiProgramFile(self, dotnet_service, api_copy_folder):
        api_program_path = os.path.join(api_copy_folder,
            'src',
            'Program.cs')

        self.csharp_templater.HandleCSharpLogging(dotnet_service,api_program_path)
    def HandleDotnetApiDbContext(self, dotnet_service, api_copy_folder):
        dbcontext_path = os.path.join(api_copy_folder,
            'src',
            'Data',
            'NameContext.cs')
        CamelCaseDbName = to_camelcase(dotnet_service['name']).replace('.','') + 'Context'
        if 'database' in dotnet_service:
            if os.path.exists(dbcontext_path):
                dbcontext_rename_path = os.path.join(api_copy_folder,
                    'src',
                    'Data',
                    CamelCaseDbName+'.cs')
                shutil.copy(dbcontext_path, dbcontext_rename_path)
                os.remove(dbcontext_path)
                replaceDict = {
                    'NameContext': CamelCaseDbName
                }
                replace_template_file(dbcontext_rename_path,replaceDict)
        else:
            remove_data_folder_path = os.path.join(api_copy_folder,
                'src',
                'Data')
            shutil.rmtree(remove_data_folder_path)
            rm_files = ['migrations.sh','updatedb.sh','dropdb.sh','migrations.dev.sh','updatedb.dev.sh','dropdb.dev.sh']
            for rm_file in rm_files:
                rm_path = os.path.join(api_copy_folder,rm_file)
                os.remove(rm_path)
            docker_file = os.path.join(api_copy_folder,
                'Dockerfile')
            filter_region_with_tag(docker_file,'database')
    
    def HandleDotnetApiNameSpaceAndCleaning(self, dotnet_service, api_copy_folder):
        src_path = os.path.join(api_copy_folder,'src')
        file_clean_paths = FindAllFilesWithExtensionInDirectory(src_path,('.cs','.csproj'))
        CamelCaseServiceName = to_camelcase(dotnet_service['name'])
        self.csharp_templater.ReplaceDotnetNameSpaces(file_clean_paths,'DotnetWebApi',CamelCaseServiceName)
        ClearRegionLines(file_clean_paths)
    def HandleDotnetApiDockerFile(self, dotnet_service, api_copy_folder):
        docker_file_path = os.path.join(api_copy_folder,'Dockerfile')
        docker_replace_dict = {}
        docker_replace_dict['{{port}}'] = str(dotnet_service['port'])
        docker_replace_dict['{{project_name}}'] = to_camelcase(dotnet_service['name'])
        replace_template_file(docker_file_path,docker_replace_dict)
        if 'database' in dotnet_service:
            ef_shell_replace_dict = {
                '{{ProjectName}}' : to_camelcase(dotnet_service['name']),
                '{{DatabaseContextName}}' : to_camelcase(dotnet_service['name']).replace('.','') + 'Context'
            }
            shell_file_paths = ['migrations.sh','updatedb.sh','dropdb.sh','migrations.dev.sh','updatedb.dev.sh','dropdb.dev.sh']
            for path in shell_file_paths:
                f_path = os.path.join(api_copy_folder,path)
                replace_template_file(f_path,ef_shell_replace_dict)

    def HandleDotnetApiDockerCompose(self, dotnet_service,api_copy_folder):
        docker_props = {
            'image': dotnet_service['name'].lower(),
            'build': {
                'context': 'src/ApiServices/'+to_camelcase(dotnet_service['name'])+'/',
                'dockerfile': 'Dockerfile'
            },
            'restart': 'on-failure',
            'ports': [],
            'links': [],
            'depends_on':[],
            'networks':['localnet'],        
        }
        if 'database' in dotnet_service:
            docker_props['links'].append(dotnet_service['database']['provider'])
            docker_props['depends_on'].append(dotnet_service['database']['provider'])
        if 'port' in dotnet_service:
            docker_props['ports'].append(str(dotnet_service['port'])+':'+str(dotnet_service['port']))
        eventbus_enabled = 'eventbus' in dotnet_service
        if eventbus_enabled:
            eb_provider = dotnet_service['eventbus']['provider']        
            docker_props['links'].append(eb_provider)
            docker_props['depends_on'].append(eb_provider)

        return docker_props
    


from modules.basemodule import BaseModule
from modules.devops.docker import Docker
import os
class Postgre(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath):
        super().__init__(projectOptions, project_templates_paths, outputPath)

    def HandlePostgre(self, db_options):
        docker_options = self.BuildDockerOptions(db_options)
        docker_instance = Docker.getInstance()
        docker_instance.AddService(db_options['name'],docker_options)
        docker_instance.AddVolume('postgres-volume')

    def BuildDockerOptions(self, db_options):    
        docker_volume_dir = os.path.normpath(os.path.join(self.outputPath,'docker_volumes','postgresql'))
        if not os.path.isdir(docker_volume_dir):
            os.makedirs(docker_volume_dir)
        default_postgre_options = {
            'image': 'postgres',
            'container_name': db_options['name'],
            'volumes': ['postgres-volume:'+docker_volume_dir],
            'networks':['localnet'],
            'ports': ['5432:5432'],
            'environment': {
                'POSTGRES_DB': 'dev',
            }            
        }
        
        if 'docker_compose_override' in db_options:
            default_postgre_options.update(db_options['docker_compose_override'])  
        if 'username' in db_options:
            default_postgre_options['environment']['POSTGRES_USER'] = db_options['username']
        if 'password' in db_options:
            default_postgre_options['environment']['POSTGRES_PASSWORD'] = db_options['password']
        
        return default_postgre_options

    
def BuildPostgreConnectionString(server_host,database_name,user,password):       
    conn_string ="Server={0};Database={1};Username={2};Password={3}".format(server_host,database_name,user,password)
    conn_string_dev = "Server={0};Database={1};Username={2};Password={3}".format('localhost',database_name,user,password)
    return conn_string, conn_string_dev



import json
import re
import os

def InDbQ(value):
    return '\"'+value+'\"'

def to_camelcase(s):
    s_val = re.sub(r'(?!^)_([a-zA-Z])', lambda m: m.group(1).upper(), s)
    return s_val[0].upper()+s_val[1:]

def FindClientWithName(projectOptions,name):
    clients = projectOptions['clients']
    for client in clients:
        if list(client.values())[0]['name'] == name:
            return list(client.values())[0]

def FindIdentityServiceWithName(projectOptions,name):
    identity_services = projectOptions['identity_services']
    for i_s in identity_services:
        if list(i_s.values())[0]['name'] == name:
            return list(i_s.values())[0]

def FindDatabaseWithName(projectOptions,name):
    database_instances = projectOptions['databases']
    for db in database_instances:
        if list(db.values())[0]['name'] == name:
            return list(db.values())[0]

def FindEventBusWithName(projectOptions,name):
    eventbus_instances = projectOptions['eventbus']
    for bus in eventbus_instances:
        if list(bus.values())[0]['name'] == name:
            return list(bus.values())[0]

def FindServerWithName(projectOptions,name):
    server_instances = projectOptions['servers']
    for server in server_instances:
        if list(server.values())[0]['name'] == name:
            return list(server.values())[0] 




def FindAllFilesWithExtensionInDirectory(folder_path, extensions):
    ext_files = []
    for folder_root, dirs, files in os.walk(folder_path):
        for file in files:
            if file.endswith(extensions):
                ext_files.append(os.path.join(os.path.abspath(folder_root),file))
    return ext_files



from modules.basemodule import BaseModule
class Server(BaseModule):
    def __init__(self, projectOptions, project_templates_paths, outputPath, server_options):
        self.server_options = server_options
        super().__init__(projectOptions,project_templates_paths,outputPath)
    def FindServerWithName(self,name):
        server_instances = self.projectOptions['servers']
        for server in server_instances:
            if list(server.values())[0]['name'] == name:
                return list(server.values())[0] 